 /****************************************************************************\
*  RTC Demo                                                                    *
*  Copyright 2004 Timmy Brolin                                                 *
*                                                                              *
*  This program is free software; you can redistribute it and/or modify        *
*  it under the terms of the GNU General Public License as published by        *
*  the Free Software Foundation; either version 2 of the License, or           *
*  (at your option) any later version.                                         *
*                                                                              *
*  This program is distributed in the hope that it will be useful,             *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of              *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
*  GNU General Public License for more details.                                *
*                                                                              *
*  You should have received a copy of the GNU General Public License           *
*  along with this program; if not, write to the Free Software                 *
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   *
 \****************************************************************************/

// This demo is intended to show how to access the RTC (Real Time Clock) in certain 
// Gameboy Advance cartridges, such as Pokemon cartridges, the X-ROM flash cartridge,
// and the F2A Ultra flash cartridge.
// To try this demo on a Pokemon cartridge, simply put the Pokemon cartridge in your 
// Gameboy Advance and multiboot this demo.

// This demo does not work with EZ RTC flash cartridges, simply because the RTC in 
// thoose cartridges is non-standard and incompatible. It is currently unknown how
// to access the EZ RTC.

#include <stdlib.h>
#include "gba.h"
#include "screenmode.h"
#include "font.h"
#include "keypad.h"
#include "rtc.h"

int charpos=0;

// Is the screen in vertical blank?
u16 scrIsVbl(){
	return( REG_VCOUNT > 160 );
}
// Wait until vertical blank
void scrWaitVbl(){
	while( !scrIsVbl() );
}
// Wait for NEXT vertical blank
void scrWaitVSync(){
	while( scrIsVbl() );
	while( !scrIsVbl() );
}

//Writes one character to screen
void typechar(unsigned char c){
    if(c){
        if(c==8){ //backspace
            charpos--;
            VideoBuffer[0x4000+charpos]=' ';
        }else{
            VideoBuffer[0x4000+charpos]=c;
            charpos++;
        }
    }
}

//Writes one byte to screen in hex format
void typehex(unsigned char c){
unsigned char hi, lo, ch;
    hi = c>>4;
    if(hi<10){
        ch=hi+0x30;
    }else{
        ch=hi+0x41-10;
    }
    VideoBuffer[0x4000+charpos]=ch;
    charpos++;
    
    lo = c & 0x0F;
    if(lo<10){
        ch=lo+0x30;
    }else{
        ch=lo+0x41-10;
    }
    VideoBuffer[0x4000+charpos]=ch;
    charpos++;
}

//Writes one string to screen
void typestr(char* data){
int ptr=0;
    while(data[ptr]!=0){
        typechar(data[ptr]);
        ptr++;
    }
}

//Main
int main(void){
    int loop, x_loop, y_loop;
	char day_of_week[7][4]={"mon", "tue", "wed", "thu", "fri", "sat", "sun"};

    //Load font palette
    for(loop = 0; loop < fontPalLen; loop++)
	    BGPaletteMem[loop] = fontPal[loop];
    //Load font characters
    for(loop=0; loop<fontDataLen; loop++)
        VideoBuffer[loop]=fontData[loop];
    //Fill screen with spaces
    for(y_loop=0; y_loop<20; y_loop++)
        for (x_loop=0; x_loop<30; x_loop++)
            VideoBuffer[0x4000+x_loop+32*y_loop]=' ';

    SetMode(MODE_0 | BG0_ENABLE);
    REG_BG0CNT = BG_COLOUR_256x1 | BG_T256x256 |BG_MAPBASE(0x8000) | BG_CHARBASE(0x0000) | BG_PRIORITY(3);

    charpos=64;
    typestr("  RTC Demo by Timmy Brolin");
	charpos=192;
	if(rtc_detect())
		typestr("  RTC Detected.");
	else{
		typestr("  No RTC Detected.");
		while(1);
	}
	//Main loop
	while(1){
		charpos=258;
		scrWaitVSync();

		//Read date and time from RTC and put it on the display.
		rtc_init_transfer();
		//Year
		typestr("20");
		typehex(rtc_read());
		typestr("/");
		//Month
		typehex(rtc_read());
		typestr("/");
		//Date
		typehex(rtc_read());
		typestr(" ");
		//Day of week
		typestr(day_of_week[(int)rtc_read()]);
		typestr(" ");
		//Hour
		typehex(rtc_read()&0x7F);
		typestr(":");
		//Minute
		typehex(rtc_read());
		typestr(":");
		//Second
		typehex(rtc_read());
		rtc_end_transfer();
	}
}
